'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.onPluginLoad = onPluginLoad;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _parser = require('@babel/parser');

var babelParser = _interopRequireWildcard(_parser);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Default babel parser options applying most available plugins.
 *
 * Caveats include:
 * - that both decorators and decorators-legacy can not be used simultaneously.
 * - that both 'flow' and 'typescript' can not be used simultaneously
 *
 * @type {{plugins: string[]}}
 * @ignore
 */
var s_DEFAULT_BABELPARSER_OPTIONS = {
  plugins: ['asyncGenerators', 'bigInt', 'classProperties', 'classPrivateProperties', 'classPrivateMethods', ['decorators', { decoratorsBeforeExport: false }], 'doExpressions', 'dynamicImport', 'exportDefaultFrom', 'exportNamespaceFrom', 'functionBind', 'functionSent', 'importMeta', 'jsx', 'logicalAssignment', 'nullishCoalescingOperator', 'numericSeparator', 'objectRestSpread', 'optionalCatchBinding', 'optionalChaining', ['pipelineOperator', { proposal: 'minimal' }], 'throwExpressions', 'typescript']
};

/**
 * Provides a convenience wrapper around Babel parser.
 */

var BabelParser = function () {
  function BabelParser() {
    (0, _classCallCheck3.default)(this, BabelParser);
  }

  (0, _createClass3.default)(BabelParser, null, [{
    key: 'parseFile',

    /**
     * Load and parse source code with Babel parser.
     *
     * @param {string}   filePath - source code file path.
     * @param {object}   [options] - Overrides default babel parser options.
     * @param {object}   [override] - Provides helper directives to override options to simplify modification of default
     *                                Babel parser options.
     *
     * @returns {object} AST of source code.
     */
    value: function parseFile(filePath) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : void 0;
      var override = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : void 0;

      return BabelParser.parseSource(_fs2.default.readFileSync(filePath, { encode: 'utf8' }).toString(), options, override);
    }

    /**
     * Parses the given source with Babel parser.
     *
     * @param {string}   source - Javascript source code to parse.
     * @param {object}   [options] - Overrides default babel parser options.
     * @param {object}   [override] - Provides helper directives to override options to simplify modification of default
     *                                Babel parser options.
     *
     * @returns {object}
     */

  }, {
    key: 'parseSource',
    value: function parseSource(source) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : void 0;
      var override = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : void 0;

      options = (typeof options === 'undefined' ? 'undefined' : (0, _typeof3.default)(options)) === 'object' ? options : JSON.parse((0, _stringify2.default)(s_DEFAULT_BABELPARSER_OPTIONS));
      options.sourceType = typeof options.sourceType === 'string' ? options.sourceType : 'unambiguous';

      if ((typeof override === 'undefined' ? 'undefined' : (0, _typeof3.default)(override)) === 'object' && (0, _typeof3.default)(Array.isArray(options.plugins))) {
        // If flow is enabled as an override 'typescript' must be removed and 'flow' added.
        if (typeof override.flow === 'boolean' && override.flow) {
          var index = options.plugins.indexOf('typescript');
          if (index > -1) {
            options.plugins.splice(index, 1);
          }

          options.plugins.push('flow');
        }
      }

      return babelParser.parse(source, options);
    }
  }]);
  return BabelParser;
}();

/**
 * Wires up BabelParser on the plugin eventbus. The following event bindings are available:
 *
 * `typhonjs:babel:parser:file:parse`: Invokes `parseFile`.
 * `typhonjs:babel:parser:source:parse`: Invokes `parseSource`.
 *
 * @param {PluginEvent} ev - The plugin event.
 * @ignore
 */


exports.default = BabelParser;
function onPluginLoad(ev) {
  var eventbus = ev.eventbus;

  eventbus.on('typhonjs:babel:parser:file:parse', BabelParser.parseFile, BabelParser);
  eventbus.on('typhonjs:babel:parser:source:parse', BabelParser.parseSource, BabelParser);
}